require 'savon'

module Omniva
  module API
    class Client
      def self.call(data = nil)
        validate_config
        data ||= initial_request
        client = Savon.client do
          wsdl Omniva::API.config[:wsdl]
          open_timeout Omniva::API.config[:timeout].to_i
          read_timeout Omniva::API.config[:timeout].to_i
        end
        auth = { authPhrase: Omniva::API.config[:key] }
        response = client.call :single_address1, attributes: auth, message: data
        parse_response(response)
      end

      def self.validate_config
        messages = {
          wsdl: 'WSDL is missing',
          key: 'Key is missing',
          timeout: 'Timeout is missing'
        }

        messages.each do |key, value|
          fail InvalidConfiguration, value if Omniva::API.config[key].blank?
        end
      end

      def self.parse_response(response)
        unwrapped =
          response
          .to_hash
          .fetch(:single_address_response)
          .fetch(:aadress_komponent_nimekiri)
        unwrapped ? unwrapped.fetch(:aadress_komponent) : {}
      end

      def self.initial_request
        { nimetus: 'Eesti' }
      end
    end
  end
end
